/*
 * OpenI2CRADIO
 * Menu sub-routines.
 * Copyright (C) 2013-06-21 K.Ohta <whatisthis.sowhat ai gmail.com>
 * License: GPL2+LE
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2,
 *  or (at your option) any later version.
 *  This library / program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this library; see the file COPYING. If not, write to the
 *  Free Software Foundation, 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301, USA.
 *
 *  As a special exception, if you link this(includeed from sdcc) library
 *  with other files, some of which are compiled with SDCC,
 *  to produce an executable, this library does not by itself cause
 *  the resulting executable to be covered by the GNU General Public License.
 *  This exception does not however invalidate any other reasons why
 *  the executable file might be covered by the GNU General Public License.
 */

#include "menu.h"
#include "power.h"
#include "commondef.h"


void toggle_amfm(void)
{
    unsigned int freq;
    if(fm != 0){
        fm = 0;
        fmfreq_bank[fmband] = fmfreq;
  //      akc6955_set_amband(amband);
        freq = amfreq_bank[amband];
    } else {
        fm = 0xff;
        amfreq_bank[amband] = amfreq;
//        akc6955_set_fmband(fmband);
        freq = fmfreq_bank[fmband];
    }
    akc6955_chg_fm(fm, freq);
    idle_time_ms(500);
//    akc6955_set_freq(freq);
}


void set_thresh_fmstereo(unsigned char a)
{
    unsigned char b;
    a = a & 0x03;
    threshold_fmstereo = a;
    b = akc6955_readcmd(AKC6955_THRESH) & 0xfc;
    akc6955_writecmd(AKC6955_THRESH, a | b);
}

void set_thresh_width(unsigned char a)
{
    unsigned char b;
    a = a & 0x03;
    threshold_width = a;
    a = a << 2; // << 2
    b = akc6955_readcmd(AKC6955_THRESH) & 0xf3;;
    akc6955_writecmd(AKC6955_THRESH, a | b);
}

void set_thresh_amcnr(unsigned char a)
{
    unsigned char b;
    a = a & 0x03;
    threshold_amcnr = a;
    a = a << 4; // << 4
    b = akc6955_readcmd(AKC6955_THRESH) & 0xcf;
    akc6955_writecmd(AKC6955_THRESH, a | b);
}

void set_thresh_fmcnr(unsigned char a)
{
    unsigned char b;
    a = a & 0x03;
    threshold_fmcnr = a;
    a = a << 6; // << 4
    b = akc6955_readcmd(AKC6955_THRESH) & 0x3f;
    akc6955_writecmd(AKC6955_THRESH, a | b);
}

void set_stereomode(void)
{
    __bitops_t mode;
    mode.byte = 0x00;
    if(stereo == 0x00){
            mode.b0 = 1; // Force mono
    } else if(stereo < 0x80){
            mode.b1 = 1; // Force stereo
    }
    akc6955_set_stereomode(mode.byte);
}

void set_stereo(void)
{
    
    if(stereo == 0xff) {// Force Mono
        stereo = 0x00;
    } else if(stereo == 0x00) { //Mono->Stereo
        stereo = 0x7f;
    } else { // Default, Stereo->auto
        stereo = 0xff;
    }
    set_stereomode();
}


void set_volume(void)
{
    unsigned char c;
    unsigned char d;
    unsigned char p;
    unsigned char fact;

    _CLS();
    do {
    _LOCATE(0,0);
     d = akc6955_getvolumemode();
      if(d == 0){
         _PUTCHAR('D');
       } else {
         _PUTCHAR('K');
       }
     printstr(" Vol:");
     print_numeric_nosupress(volume, 2);
     printstr("  F:Exit");
     _LOCATE(0,1);
     printstr("Pre:");
     _PUTCHAR('0' + (prevolume & 3));
     _LOCATE(16-4, 1);
     if(lowboost == 0x00){
         printstr("   ");
     } else {
         printstr("LOW");
     }
     _HOME();
     c = pollkeys(pollkeybuf, 60, 1);
     p = 0;
     while(c > 0) {
         switch(pollkeybuf[p]) {
             case charcode_6:
                 volume++;
                 if(volume > 63) volume = 63;
                 break;
             case charcode_4:
                 volume--;
                 if(volume < 23) volume = 23;
                 break;
             case charcode_f:
                 _CLS();
                 return;
             case charcode_a:
                 fact = 0xff;
                 if(d != 0){
                     fact = 0x00;
                 }
                 akc6955_setvolumemode(fact);
                 break;
             case charcode_b:
                 prevolume++;
                 if(prevolume > 3) prevolume = 0;
                 akc6955_set_prevolume(prevolume);
                 break;
             case charcode_d:
                 fact = 0x00;
                 if(lowboost == 0x00){
                     fact = 0xff;
                 }
                 lowboost = fact;
                 akc6955_set_lowboost(lowboost);
                 break;
             default:
                 break;
         }
         c--;
         p++;
        }
     if(volume < 24){
         set_examp_mute(1);
     } else {
         set_examp_mute(0);
     }
     akc6955_setvolume(volume);
//    _HOME();
    } while(1);
}

void scan_start(void)
{
    unsigned char c;
    _CLS();
    _LOCATE(0,0);
    printstr("Scan F/A/4/6");
    do {
        update_status();
        print_freq(1);
        c = pollkey_single();
       // New Scan
       if(c == charcode_6){
           scanflag = 0xff;
           akc6955_do_scan(0xff);
       } else if(c == charcode_4){
           scanflag = 0xff;
           akc6955_do_scan(0);
       } else if(c == charcode_a){
           scanflag = 0;
           akc6955_abort_scan();
           break;
       } else if(c == charcode_f){
           break;
       } else {
           if((scanflag != 0) && (akc6955_chk_donescan() != 0)) {
                scanflag = 0;
                _HOME();
                printstr("Scan F/A/4/6");
                update_status();
                print_freq(1);
           }
       }
    _HOME();
    } while(1);
    _CLS();
}

void setfreq_direct(void)
{
    unsigned int val;
    _CLS();
    _LOCATE(0,0);
    printstr("Set Freq:");
    _LOCATE(0,1);
    if(fm != 0){
        // FM
        printstr("FM ");
        val = fmfreq;
        val = read_numeric(val, 5, 7, 1);
        fmfreq = val;
    } else {
        // FM
        printstr("AM ");
        val = amfreq;
        val = read_numeric(val, 5, 7, 1);
        amfreq = val;
    }
    akc6955_set_freq(val);
}

void setband_direct(void)
{
    unsigned int band;
    _CLS();
    _LOCATE(0,0);
    if(fm != 0){
        printstr("Set Band:FM#");
        band = fmband & 7;
        fmfreq_bank[fmband] = amfreq;
        fmband = read_numeric(band, 2, 7, 1);
        akc6955_set_fmband(fmband);
        akc6955_do_tune();
        fmfreq = fmfreq_bank[fmband];
        akc6955_set_freq(fmfreq);
    } else {
        printstr("Set Band:AM#");
        band = amband & 0x1f;
        amfreq_bank[amband] = amfreq;
        amband = read_numeric(band, 2, 7, 1);
        amfreq = amfreq_bank[amband];
        akc6955_set_amband(amband);
        akc6955_do_tune();
        akc6955_set_freq(amfreq);
    }
}

void call_userband(unsigned char num)
{
    unsigned int freq;
    unsigned int ch;
    unsigned char start;
    unsigned char end;
    unsigned char mode3k;
    unsigned char freq2;
    _userband_t *p;

    if(num >= USER_BAND_NUM) return;
    
    p = &(am_usrbands[num]);
    if(fm != 0){
        p = &(fm_usrbands[num]);
    }
    freq = p->freq;
    start = p->start;
    end = p->stop;
    mode3k = p->mode3k;
    if(fm != 0) {
        ch = ((freq - 3000) / 5) * 2;
        fmband = AKC6955_BAND_FMUSER;
    } else {
        unsigned int q = 5;
        if(mode3k != 0) q = 3;
        ch = freq / q;
        amband = AKC6955_BAND_AMUSER;
    }

    if(start > end) {
        unsigned char tmp;
        tmp = start;
        start = end;
        end = tmp;
    }
    
    freq2 = ch / 32;
    if(freq2 > end) {
        ch = end * 32;
    }
    if(freq2 < start){
        ch = start * 32;
    }
    akc6955_set_userband(start, end, ch, mode3k);
}

void set_userband(void)
{
    unsigned int from,to;
    unsigned char c;
    unsigned char p;
    unsigned char mode3k;
    char cc;

    _CLS();
    _LOCATE(0,0);
    printstr("User ch:");
    c = pollkey_single();

    if(c > charcode_0) return;
    if(c < charcode_1) return;
    if(c == charcode_0) {
        c = 0;
    } else {
        c = c - charcode_1 + 1;
    }
    if(c >= USER_BAND_NUM) return;
    if(fm != 0){
        from = fm_usrbands[c].start * 80 + 3000; // 32*25/10
        to = fm_usrbands[c].stop * 80 + 3000;
        _CLS();
        _LOCATE(0,0);
        _LOCATE(0,0);
        printstr("FM#");
        print_numeric_nosupress(c, 1);
        _LOCATE(0,1);
        printstr(" From:");
        from = read_numeric(from, 5, 7, 1);
        _LOCATE(0,1);
        printstr(" To:  ");
        to = read_numeric(to, 5, 7, 1);
        fm_usrbands[c].start = (from - 3000) / 80;
        fm_usrbands[c].stop = (to - 3000) / 80;
        fm_usrbands[c].freq = from * 80 + 3000;
        fm_userbandnum = c;
    } else {
        mode3k = am_usrbands[c].mode3k;
        p = 96; // 3*32
        if(mode3k == 0) p = 160; // 5*32
        from = am_usrbands[c].start * p;
        to = am_usrbands[c].stop * p;
        _CLS();
        _LOCATE(0,0);
        printstr("AM#");
        print_numeric_nosupress(c, 1);
        printstr(" Step:");
        _LOCATE(0,1);
        printstr("0=3k 1=5k");
        cc = pollkey_single();
        if(cc == charcode_0){
            p = 96;
            mode3k = 0xff;
        } else if(cc = charcode_1) {
            p = 160;
            mode3k = 0;
        }
        _CLS();
        _LOCATE(0,0);
        printstr("AM#");
        print_numeric_nosupress(c, 1);
        _LOCATE(0,1);
        printstr(" From:");
        from = read_numeric(from, 5, 7, 1);
        _LOCATE(0,1);
        printstr(" To:  ");
        to = read_numeric(to, 5, 7, 1);
        am_usrbands[c].start = from / p;
        am_usrbands[c].stop = to  / p;
        am_usrbands[c].mode3k = mode3k;
        am_usrbands[c].freq = from * p;
        am_userbandnum = c;
    }
    call_userband(c);
    _CLS();
}

void input_userband(void)
{
    unsigned char c;
    do{
    _CLS();
    _LOCATE(0,0);
    printstr("User Band");
    _LOCATE(0,1);
    printstr("   #");
    c = pollkey_single();

    if((c >= charcode_a) && (c <= charcode_f)){
        break;
    }
    if(c == charcode_0) {
        _PUTCHAR('0');
        if(fm != 0){
           fm_userbandnum = 0;
        } else {
           am_userbandnum = 0;
        }
        call_userband(0);
    } else {
        c = c - charcode_1 + 1;
        if(c < USER_BAND_NUM) {
            _PUTCHAR(c + '0');
            if(fm != 0){
               fm_userbandnum = c;
            } else {
                am_userbandnum = c;
            }
            call_userband(c);
        }
    }
    idle(ui_idlecount);
    } while(1);
    _CLS();
}


void menu_poweroff(void)
{
    unsigned char c;
    c = printhelp_2lines("Poweroff", "A=Yes");
    if(c == charcode_a) {
        shutdown(1);
    }
}

void setup_akc6955(void)
{
//    akc6955_chg_fm(fm); // Set to AM
#if 0
/*
 * CR_DEF[18]= {
            0x4c,  //REG00    //Modify
            0x10,  //REG01
            0x4a,  //REG02
            0xc8,  //REG03
            0x19,  //REG04
            0x32,  //REG05
            0xa1,  //REG06
            0xa1,  //REG07
            0x54,  //REG08
            0xC7,  //REG09
            0x7f,  //REG10
            0xe0,  //REG11
            0x00,  //REG12
            0x0C,  //REG13
            0x40,  //REG14
            0x82,  //REG15        //Modify
            0xcc,  //REG16
            0xf9   //REG17
 */
    akc6955_writecmd(0x00, 0x4c);
    akc6955_writecmd(0x01, 0b00010000);
    akc6955_writecmd(0x02, 0b01001010);
    akc6955_writecmd(0x03, 0xc8);
    akc6955_writecmd(0x04, 0x19);
    akc6955_writecmd(0x05, 0x32);
    akc6955_writecmd(0x06, 0xa1);
    akc6955_writecmd(0x07, 0b10100001);
    akc6955_writecmd(0x08, 0b01010000);
    akc6955_writecmd(0x09, 0b11000111);
    akc6955_writecmd(0x09, 0b01111111);
    akc6955_writecmd(0x0b, 0b11100000);
    akc6955_writecmd(0x0c, 0b00000000);
    akc6955_writecmd(0x0d, 0b00001100);
    akc6955_writecmd(0x0e, 0x40);
    akc6955_writecmd(0x0f, 0x82);
    akc6955_writecmd(0x10, 0xcc);
    akc6955_writecmd(0x11, 0xf9);


#else
    akc6955_writecmd(AKC6955_POWER, 0xc0);  // You musto *not* mute, set b2 to '0".
    akc6955_writecmd(AKC6955_VOLUME, 0xc0); // You must select to radio(b1 = '0).
    if(fm == 0) {
        akc6955_set_amband(amband);
        if(amband == AKC6955_BAND_AMUSER) call_userband(am_userbandnum);
        amfreq = amfreq_bank[amband];
        akc6955_chg_fm(0, amfreq); // Set to AM
        akc6955_set_freq(amfreq);
    } else {
        akc6955_set_fmband(fmband);
        if(fmband == AKC6955_BAND_FMUSER) call_userband(fm_userbandnum);
        fmfreq = fmfreq_bank[fmband];
        akc6955_chg_fm(0xff, fmfreq); // Set to AM
        akc6955_set_freq(fmfreq);
    }// Dummy, TBS (954KHz)
    akc6955_set_power(0xff); // Power ON
    akc6955_setvolume(36); // Temporally
    set_thresh_fmcnr(threshold_fmcnr);
    set_thresh_amcnr(threshold_amcnr);
    set_thresh_width(threshold_width);
    set_thresh_fmstereo(threshold_fmstereo);
    akc6955_setvolumemode(0);
    akc6955_set_lowboost(lowboost);
    set_stereomode();
#endif
}

void menu_save(void)
{
    unsigned char c;
    c = printhelp_2lines("Save settings", "A=Yes");
    if(c == charcode_a) {
        save_eeprom();
    }
}

void menu_load(void)
{
    unsigned char c;
    c = printhelp_2lines("Load settings", "A=Yes B=Init");
    if(c == charcode_a) {
        c = load_eeprom();
        if( c != 0xff) {
            _CLS();
            _LOCATE(0,0);
            c = printhelp_2lines("X) Load Error", "A=Fix");
            setdefault();
            if(c == charcode_a){
                save_eeprom();
            }
        }
        setup_akc6955();
    } else if(c == charcode_b){
        setdefault();
        setup_akc6955();
    }
}

void setup_menu(void)
{
    unsigned char c;
    unsigned int val;

    c = printhelp_2lines("Setup F=HELP", "5=Return");
    switch(c){
        case charcode_3:
            _CLS();
            _LOCATE(0,0);
            printstr("BL Level:");
            val = read_numeric(backlight_level, 3, 0,1);
            if(val > 255) val = 255;
            if(val < 10) val = 15;
            backlight_level = (unsigned char)val;
            break;
        case charcode_4:
            _CLS();
            _LOCATE(0,0);
            printstr("FM Bandwidth:");
            akc6955_get_fmbandwidth(val);
            val = read_numeric(val, 1, 0, 1) & 3;
            fmbandwidth = (unsigned char)val;
            akc6955_set_fmbandwidth(fmbandwidth);
            break;
        case charcode_5:
            break;
        case charcode_7:
            _CLS();
            _LOCATE(0,0);
            printstr("FM-CNR threshold:");
            val = threshold_fmcnr;
            val = read_numeric(val, 1, 0, 1);
            set_thresh_fmcnr((unsigned char)val);
            break;
        case charcode_8:
            _CLS();
            _LOCATE(0,0);
            printstr("AM-CNR threshold:");
            val = threshold_amcnr;
            val = read_numeric(val, 1, 0, 1);
            set_thresh_amcnr((unsigned char)val);
            break;
        case charcode_9:
            _CLS();
            _LOCATE(0,0);
            printstr("Stereo threshold:");
            val = threshold_fmstereo;
            val = read_numeric(val, 1, 0, 1);
            set_thresh_fmstereo((unsigned char)val);
            break;
        case charcode_f:
            setup_help();
            break;
        case charcode_0:
            menu_poweroff();
            break;
        case charcode_a:
            menu_load();
            break;
        case charcode_c:
            menu_save();
            break;
        case charcode_d:
            setdefault();
            break;
        case charcode_e:
            _CLS();
            _LOCATE(0,0);
            printstr("BL Long:");
            val = read_numeric(backlight_long, 3, 0,1);
            if(val > 999) val = 999;
            backlight_long = val;
            break;
        default:
            break;
    }
}
/*
 * Main Menu : initial-screen -> 'F'.
 */
void main_menu(void)
{
    unsigned char c;
    unsigned char p;
    unsigned char n;
    unsigned int val;

    _CLS();
    _HOME();
    _LOCATE(0,0);
    printstr("Menu:F=HELP");
    _LOCATE(0,1);
    printstr("B=CANCEL");
       do {
           n = pollkeys(pollkeybuf, 60, 1);
       } while(n == 0);
       p = 0;
       c = pollkeybuf[0];
        if(c == charcode_1){
            // Band
            setband_direct();
        } else if(c == charcode_2){
        } else if(c == charcode_3){
            // Band
            setfreq_direct();
        } else if(c == charcode_4){

        }else if(c == charcode_5){
            scan_start();
            // Scan
        } else if(c == charcode_6){

        } else if(c == charcode_7){
            menu_poweroff();
        } else if(c == charcode_8){
            // Set volume
            set_volume();
        } else if(c == charcode_9){
            _CLS();
            _LOCATE(0,0);
            printstr("Sig width:");
            val = threshold_width;
            val = read_numeric(val, 1, 0, 1);
            set_thresh_width((unsigned char)val);
            // Set NF
        } else if(c == charcode_0){
            // Setup Menu
            setup_menu();
        } else if(c == charcode_a){
            toggle_amfm();
        } else if(c == charcode_b){
            // Cancel
        } else if(c == charcode_c){
            // Set userband
            set_userband();
        } else if(c == charcode_d){
            // Call userband
            input_userband();
        } else if(c == charcode_e){
            // Backlight
            
        } else if (c == charcode_f){
            mainmenu_help();
            // HELP
        }
       _CLS();
       _LOCATE(0,0);
}

